;**************************************************
; LEGENDA: BRANA  
; ZAVIRANI AUTOMAT	= AUT
; KONCAK  ZAVIRANI	= KSZ (N.C.)
; KONCAK  OTEVIRANI	= KSO (N.C.)
; FOTOBUNKA 		= FOT (N.C.)
; OTVIRANI KLICEM 	= MAN (N.O.)
; MOTOROVY CHRANI  	= MOS (N.C.)
;**************************************************

                PROCESSOR       16F84
                RADIX           DEC
                INCLUDE         "P16F84.INC"
                ERRORLEVEL      -302
                __CONFIG         3FF3H	;Oscilator RC.

;**********************************
;		 REGISTRY
;**********************************

 ORG		0CH						;popis registru pro compilator

Time_Otvirani			res  	2			;as otvirani
Time_Zavirani  			res    	2			;as zavirani
Time_Cekej			res	2 			;ekaci doba
Time_Zpozdi_M2			res	2			;as zpodn
Time_Sirena			res	2			;sirena
Zavreno_Otevreno		res	1			;stav zaveno/oteveno

;***********************************
;	RESET VECTOR
;***********************************
		
 ORG		00H						;Inicializace programu.
			   
Inicializace		
	bsf	STATUS,RP0					;nastav banku
	movlw	00000000B					;trasfer do W  00000000
	movwf	TRISA						;set  Port A (output)
	movlw	11111111B					;trasfer do W 11111111
	movwf	TRISB						;set  Port B (input) 
	bcf	STATUS,RP0					;nastav banku
	movlw	00010000B					;rele,zavira M2,RA4 PNP tranzistor
	movwf	PORTA
	clrf	Zavreno_Otevreno				;vynuluj registr zaveno/oteveno

;*************************************
;	START
;*************************************

Start	
	btfsc	PORTB,0						;jestli je IDO v 0,pesko nasledujici,eka na puls z DO
	goto	Start2						;kdy nepila 0,mona pijde od kliku?
	goto	StartOK						;jdi na StartOK,spusti cyklus,zavirat_otvirat
Start2
	btfsc	PORTB,6						;jestli je MAN v 0,pesko nasledujici,eka na puls kliek
	goto	Start						;jdi na Start
StartOK
	btfsc	Zavreno_Otevreno,0				;controla stavu,je zaveno nebo oteveno?
	goto	Zavirani					;jdi na Zavirani 
	goto 	Otvirani					;jdi na Otvirani

;*************************************
;	OTVIRANI
;*************************************

Otvirani
	movlw	00000001B					;kontrola zaveno/oteveno uloi (ZAVENO)
	movwf	Zavreno_Otevreno				;vlo 00000001 do registru Zavreno_Otevreno.
	clrf	Time_Otvirani					;vynuluj registr
	movlw	60						;time  (60/2=30sec) pro OTVIRANI.
	movwf	Time_Otvirani+1				
	clrf	Time_Sirena					;vynuluj registr
	movlw	6						;time  (6/2=3sec) pro Time_Sirena.
	movwf	Time_Sirena+1				
	movlw	00010011B		
	movwf	PORTA						;sirena a signalizace aktivni

Loop_Sirena
	decfsz	Time_Sirena,1		
	goto	Loop_Sirena
	decfsz	Time_Sirena+1,1
	goto	Loop_Sirena
	bcf	PORTA,0						;sirenu vypnout

Loop_Otvirani
	btfsc	PORTB,3						;je koncak KSO v 0?,pesko nasledujici instrukci
	goto	Cekej						;jdi na  CEKEJ.
	btfsc	PORTB,4						;je FOT v 0?,pesko nasledujici instrukci
	call	Pausa						;jdi na podprogram PAUSA
	btfsc	PORTB,7						;je MOS v 0?,pesko nasledujici instrukci
	call	Pausa						;jdi na podprogram PAUSA
	movlw	00011010B		
	movwf	PORTA						;vystupy nastavit,sviti a otvirat
	decfsz	Time_Otvirani,1
	goto	Loop_Otvirani
	decfsz	Time_Otvirani+1,1
	goto	Loop_Otvirani
	goto	Cekej						;jdi na  CEKEJ


;**************************************
;	CEKEJ , PRO AUT ZAVIRANI
;**************************************

Cekej
	movlw	00010010B					;vystup nastavit,sviti eka
	movwf	PORTA			
	clrf	Time_Cekej					;vynuluj registr
	movlw	40						;time (40/2=20sec) pro CEKEJ
	movwf	Time_Cekej+1				

Loop_Cekej
	btfsc	PORTB,1						;je AUT, automat v 0?,pesko nasledujici instrukci	
	goto	Stop						;STOP. 
	btfss	PORTB,0						;je IDO  impuls DO v 1?,pesko nasledujici instrukci
	goto	Zavirani					;jdi na ZAVIRANI.
	btfss	PORTB,6						;je MAN kliek v 1?,pesko nasledujici instrukci  
	goto	Zavirani					;jdi na ZAVIRANI.
	decfsz	Time_Cekej,1		
	goto	Loop_Cekej
	decfsz	Time_Cekej+1,1
	goto	Loop_Cekej
	goto	Zavirani					;jdi na ZAVIRANI.


;**************************************
;	ZAVIRANI
;**************************************

Zavirani
	clrf	Zavreno_Otevreno				;vynuluj registr zaveno/oteveno
	clrf	Time_Zavirani					;vynuluj registr
	movlw	50						;time (50/2=25sec) pro ZAVIRANI
	movwf	Time_Zavirani+1				
	clrf	Time_Zpozdi_M2					;vynuluj registr
	movlw	10						;time (10/2=5sec) pro zpozdeni M2.
	movwf	Time_Zpozdi_M2+1			

Loop_Zpozdi_M2
	btfsc	PORTB,7						;je MOS v 0?,pesko nasledujici instrukci
	goto	Otvirani					;jdi na  OTVIRANI.
	btfsc	PORTB,2						;je koncak KSZ v 0?,pesko nasledujici instrukci
	goto	Stop						;jdi na STOP. 
	btfsc	PORTB,4						;je FOT v 0?,pesko nasledujici instrukci
	goto    Otvirani					;jdi na OTVIRANI.
	movlw	00010110B		
	movwf	PORTA						;sviti M1 zavira	
	decfsz	Time_Zpozdi_M2,1		
	goto	Loop_Zpozdi_M2
	decfsz	Time_Zpozdi_M2+1,1
	goto	Loop_Zpozdi_M2

Loop_Zavirani
	btfsc	PORTB,7						;je MOS v 0?,pesko nasledujici instrukci
	goto	Otvirani					;jdi na OTVIRANI.
	btfsc	PORTB,2						;je koncak KSZ v 0?,pesko nasledujici instrukci
	goto	Stop						;jdi na STOP. 
	btfsc	PORTB,4						;je  FOT v 0?,pesko nasledujici instrukci
	goto    Otvirani					;jdi na OTVIRANI.
	movlw	00000110B		
	movwf	PORTA						;sviti M2 zavira
	decfsz	Time_Zavirani,1		
	goto	Loop_Zavirani
	decfsz	Time_Zavirani+1,1
	goto	Loop_Zavirani
	goto	Stop						;jdi na  STOP.

;*************************************
;	PAUSA
;*************************************
Pausa
	movlw	00010010B		
	movwf	PORTA						;sviti a eka na splnn podmnek
	btfsc	PORTB,4						;je FOT v 0?,pesko nasledujici instrukci
	goto	Pausa					
	btfsc	PORTB,7						;je MOS v 0?,pesko nasledujici instrukci
	goto	Pausa					
	return							;navrat z podprogramu zpt

;*************************************
;	STOP
;*************************************
Stop
	movlw	00010000B					;vechna RELE vypnout
	movwf	PORTA
	goto 	Start						;jdi na START

	END
